# Generated by Django 4.2 on 2025-04-08 05:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0024_remove_incursion_constellation_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='incursion',
            old_name='migration_constellation',
            new_name='constellation',
        ),
        migrations.RenameField(
            model_name='incursion',
            old_name='migration_infested_solar_systems',
            new_name='infested_solar_systems',
        ),
        migrations.RenameField(
            model_name='incursion',
            old_name='migration_staging_solar_system',
            new_name='staging_solar_system',
        ),
    ]
