# Generated by Django 4.2 on 2025-04-05 11:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0118_alter_skill_id'),
        ('incursions', '0019_incursion_migration_constellation_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='incursion',
            name='migration_infested_solar_systems',
            field=models.ManyToManyField(blank=True, related_name='+', to='corptools.mapsystem'),
        ),
    ]
