# Generated by Django 4.2 on 2025-04-09 05:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('incursions', '0025_rename_migration_constellation_incursion_constellation_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='badge',
            name='color',
            field=models.CharField(choices=[('Red', 'Red'), ('Green', 'Green'), ('Blue', 'Blue'), ('Yellow', 'Yellow'), ('Purple', 'Purple'), ('Neutral', 'Neutral'), ('Cyan', 'Cyan')], default='Neutral', max_length=16, verbose_name='Badge Color'),
        ),
        migrations.AddField(
            model_name='badge',
            name='letter',
            field=models.CharField(default='?', max_length=1, verbose_name='Letter'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='badge',
            name='order',
            field=models.SmallIntegerField(default=0, help_text='Order in which the badge will be displayed, lower is earlier', verbose_name='Order'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='badge',
            name='type',
            field=models.CharField(choices=[('Shield', 'Shield')], default='Shield', max_length=16, verbose_name='Badge Type'),
        ),
        migrations.AlterField(
            model_name='characterbadges',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='incursions_badge', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.DeleteModel(
            name='CharacterRoles',
        ),
        migrations.DeleteModel(
            name='Role',
        ),
    ]
