# Generated by Django 4.2 on 2025-04-09 06:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0026_badge_color_badge_letter_badge_order_badge_type_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='tags',
            field=models.CharField(help_text='Comma Seperated: Tags are fit specific Badges, such as First/New/Alt/Skills', max_length=255, verbose_name='Tags'),
        ),
        migrations.DeleteModel(
            name='SkillHistory',
        ),
    ]
