# Generated by Django 4.2 on 2025-04-08 05:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0023_alter_approvedskills_options_alter_badge_options_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='incursion',
            name='constellation',
        ),
        migrations.RemoveField(
            model_name='incursion',
            name='infested_solar_systems',
        ),
        migrations.RemoveField(
            model_name='incursion',
            name='staging_solar_system',
        ),
        migrations.AlterField(
            model_name='fleet',
            name='last_updated',
            field=models.DateTimeField(blank=True, verbose_name='Last Updated'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='last_seen',
            field=models.DateTimeField(verbose_name='Last Seen'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='waitlist_entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='incursions.waitlistentry', verbose_name='Waitlist Entry'),
        ),
    ]
