# Generated by Django 4.2.19 on 2025-04-01 04:38

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0014_alter_activefleet_fleet'),
    ]

    operations = [
        migrations.AlterField(
            model_name='activefleet',
            name='fleet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.fleet', verbose_name='Fleet'),
        ),
    ]
