# Generated by Django 4.2.19 on 2025-04-02 01:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0016_incursionsconfig_waitlist_webhooks_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='incursionsconfig',
            name='status_webhooks',
            field=models.ManyToManyField(blank=True, related_name='+', to='incursions.webhook', verbose_name='Destination Webhook for Incursion Updates'),
        ),
        migrations.AlterField(
            model_name='incursionsconfig',
            name='waitlist_webhooks',
            field=models.ManyToManyField(blank=True, related_name='+', to='incursions.webhook', verbose_name='Destination Webhook for Waitlist Updates'),
        ),
    ]
