# Generated by Django 4.2.19 on 2025-03-31 13:14

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0118_alter_skill_id'),
        ('incursions', '0012_rename_hull_approvedskills_ship_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='activefleet',
            options={'default_permissions': (), 'verbose_name': 'Waitlist - Fleet (Active)'},
        ),
        migrations.AlterField(
            model_name='activefleet',
            name='fleet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.fleet', verbose_name='Fleet'),
        ),
        migrations.AlterField(
            model_name='approvedskills',
            name='ship',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='corptools.eveitemtype', verbose_name='Ship'),
        ),
    ]
