# Generated by Django 4.2 on 2025-04-10 05:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0027_alter_waitlistentryfit_tags_delete_skillhistory'),
    ]

    operations = [
        migrations.AlterField(
            model_name='badge',
            name='color',
            field=models.CharField(choices=[('red', 'Red'), ('green', 'Green'), ('blue', 'Blue'), ('yellow', 'Yellow'), ('purple', 'Purple'), ('neutral', 'Neutral'), ('cyan', 'Cyan')], default='neutral', max_length=16, verbose_name='Badge Color'),
        ),
        migrations.AlterField(
            model_name='badge',
            name='type',
            field=models.CharField(choices=[('shield', 'Shield')], default='shield', max_length=16, verbose_name='Badge Type'),
        ),
    ]
