# Generated by Django 4.2 on 2025-04-05 11:36

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0118_alter_skill_id'),
        ('incursions', '0018_fleet_closed_fleet_last_updated_fleet_open_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='incursion',
            name='migration_constellation',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='corptools.mapconstellation'),
        ),
        migrations.AddField(
            model_name='incursion',
            name='migration_infested_solar_systems',
            field=models.ManyToManyField(blank=True, null=True, related_name='+', to='corptools.mapsystem'),
        ),
        migrations.AddField(
            model_name='incursion',
            name='migration_staging_solar_system',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='corptools.mapsystem'),
        ),
    ]
