# Generated by Django 4.2 on 2025-04-07 00:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0021_corptoolsmigration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('basic_waitlist', 'Waitlist Access'), ('waitlist_alts_view', 'View - all of a users Alts ! Danger?'), ('waitlist_announcements_manage', 'Announcements - Manage'), ('waitlist_badges_view', 'Badges - View'), ('waitlist_badges_manage', 'Badges - Manage'), ('waitlist_bans_view', 'Bans - View'), ('waitlist_bans_manage', 'Bans - Manage'), ('waitlist_documentation_view', 'Documentation - View'), ('waitlist_fleet_view', 'Fleet Composition'), ('waitlist_history_view', 'Fleet History'), ('waitlist_notes_view', 'Notes - View'), ('waitlist_notes_manage', 'Notes - Manage'), ('waitlist_search', 'Search'), ('waitlist_stats_view', 'Fleet Stats'), ('waitlist_skills_view', 'Skills'), ('waitlist_context_a', 'Context: Number of Pilots'), ('waitlist_context_b', 'Context: Ship Types'), ('waitlist_context_c', 'Context: Time in Waitlist'), ('waitlist_context_d', 'Context: Pilot Names'), ('waitlist_implants_view', 'Context: Implants'), ('waitlist_manage_waitlist', 'Waitlist Manage'), ('waitlist_manage_waitlist_approve_fits', 'Waitlist Manage Fits'), ('waitlist_esi_show_info', 'Show Info'), ('waitlist_esi_search', 'Search - ESI'))},
        ),
    ]
